#!/usr/bin/env python
# encoding: utf-8
"""
GMarker.py
"""
from AccessControl import ClassSecurityInfo
from ComputedAttribute import ComputedAttribute
from zope import interface

from Products.Archetypes.atapi import *
from Products.CMFCore.utils import getToolByName

from Products.ATContentTypes.content.document import ATDocument
from Products.ATContentTypes.content.schemata import finalizeATCTSchema
from Products.ATContentTypes.configuration import zconf

from Products.ATGoogleMaps.interfaces import IGMarker
from Products.ATGoogleMaps.config import *
from Products.ATGoogleMaps.field import LatLngField
from Products.ATGoogleMaps.widget import LatLngWidget

schema = Schema((
    StringField('address',
                widget=StringWidget(
                    label='Address',
                    label_msgid='label_address',
                    description_msgid='help_address',
                    i18n_domain='googlemaps',
                    size=60,
                    )
                ),
    StringField('phone',
                widget=StringWidget(
                    label='Phone',
                    label_msgid='label_phone',
                    description_msgid='help_phone',
                    i18n_domain='googlemaps',
                    size=30,
                    )
                ),
    StringField('url',
                widget=StringWidget(
                   label='URL',
                   label_msgid='label_url',
                   description_msgid='help_url',
                   i18n_domain='googlemaps',
                   size=60,
                   )
                ),
    StringField('imageurl',
                widget=StringWidget(
                    label='Image URL',
                    label_msgid='label_image_url',
                    description_msgid='help_image_url',
                    i18n_domain='googlemaps',
                    size=60,
                    )
                ),
    BooleanField('streetView',
                 widget=BooleanWidget(
                     label="Display Street View",
                     label_msgid="label_street_view",
                     description_msgid="help_street_view",
                     i18n_domain="googlemaps",
                     ),
                 ),
    LatLngField('point',
                required=True,
                default_method='getContainerCenter',
                widget=LatLngWidget(
                    label='Point',
                    label_msgid='label_point',
                    description='Point of this marker.',
                    description_msgid='help_point',
                    i18n_domain='googlemaps',
                    size=12,
                    )
                ),
    ),)
gmarker_schema = getattr(ATDocument, 'schema', Schema(())).copy() + schema.copy()
finalizeATCTSchema(gmarker_schema)

class GMarker(ATDocument):
    """
    For GMarker
    """
    security = ClassSecurityInfo()
    interface.implements(IGMarker)
    schema = gmarker_schema
    
    meta_type = "GMarker"
    _at_rename_after_creation = True
    
    # default point value from parent GMap center
    def getContainerCenter(self):
        latlng = {'latitude': '0.0', 'longitude': '0.0'}
        try:
            latlng = self.aq_parent.getCenter()
        except:
            pass
        return latlng

registerType(GMarker, PROJECTNAME)



#!/usr/bin/env python
# encoding: utf-8
"""
test_content.py

Created by Manabu Terada on 2009-12-16.
Copyright (c) 2009 CMScom. All rights reserved.
"""
import random
from Products.CMFCore.utils import getToolByName

import Products.ATGoogleMaps
import base

class TestContentTypes(base.TestCase):
    """ TopPage test """ 
    def afterSetUp(self):
        qi = self.portal.portal_quickinstaller
        qi.installProduct('ATGoogleMaps')
        self.css        = self.portal.portal_css
        self.kupu       = self.portal.kupu_library_tool
        self.skins      = self.portal.portal_skins
        self.types      = self.portal.portal_types
        self.factory    = self.portal.portal_factory
        self.workflow   = self.portal.portal_workflow
        self.properties = self.portal.portal_properties
        self.g_allow_types = [
                            "GMarker",
                            ]
        self.no_g_allow_types = [
                                # "BlogEntry",
                                # "ReportFolder",
                                ]
        self.folder_into_types = [
                    # ("contenttype name", (parent,))
                    # ("BlogEntry", ('BlogFolder',)),
                                ]
        self.addingtypes = self.g_allow_types + self.no_g_allow_types
        

    # def test_testcode_test(self):
    #     self.assertEqual(set(self.no_g_allow_types), 
    #             set((i for p, t in self.folder_into_types for i in t)))

    def test_types_installed(self):
        for t in self.addingtypes:
            self.failUnless(t in self.types.objectIds())

    def test_portal_factorySetup(self):
        for t in self.addingtypes:
            self.failUnless(t in self.factory.getFactoryTypes())

    def test_permission(self):
        permissions = [p['name'] for p in self.app.permission_settings()]
        for t in self.addingtypes:
            p = 'ATGoogleMaps: Add %s' % t
            self.failUnless(p in permissions)

    def test_global_creating(self):
        self.setRoles(['Manager'])
        allow_content_types = [al.id for al in self.portal.allowedContentTypes()]
        print allow_content_types
        for t in self.g_allow_types:
            self.failUnless(t in allow_content_types)
            id = t + '1'
            self.portal.invokeFactory(t, id)
            obj = getattr(self.portal, id)
            self.failUnless(self.portal.hasObject(id))
            # self.failUnless(obj()) No Good #######

        for t in self.no_g_allow_types:
            self.failUnless(t not in allow_content_types)
    
    def test_folder_into_creating(self):
        self.setRoles(['Manager'])
        for t, ps in self.folder_into_types:
            parent = self.portal
            for p in ps:
                base_parent = parent
                pid = p + str(random.random())
                base_parent.invokeFactory(p, pid)
                parent = getattr(base_parent, pid)
                self.failUnless(base_parent.hasObject(pid))
                self.failUnless(parent())
            allw_content_types = [al.id for al in parent.allowedContentTypes()]

            # print t, parent.id # Is this comment?
            self.failUnless(t in allw_content_types)
            tid = t + '1'
            parent.invokeFactory(t, tid)
            tobj = getattr(parent, tid)
            self.failUnless(parent.hasObject(tid))
            print tobj.absolute_url(), tobj.id
            self.failUnless(tobj())    

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestContentTypes))
    return suite




#!/usr/bin/env python
# encoding: utf-8
"""
GMap.py
"""
from AccessControl import ClassSecurityInfo
from ComputedAttribute import ComputedAttribute
from zope import interface

from Products.Archetypes.atapi import *
from Products.CMFCore.utils import getToolByName

from Products.ATContentTypes.content.folder import ATFolder
from Products.ATContentTypes.content.schemata import finalizeATCTSchema
from Products.ATContentTypes.configuration import zconf

from Products.ATGoogleMaps.interfaces import IGMap
from Products.ATGoogleMaps.config import *
from Products.ATGoogleMaps.field import LatLngField
from Products.ATGoogleMaps.widget import LatLngWidget

zoom_vocaburary = range(23)
zoom_vocaburary.reverse()

schema = Schema((
    IntegerField('height',
	        default="480",
	        required=True,
	        widget=IntegerWidget(
	            label='Height',
	            label_msgid='label_height',
	            description='Height(pixel) of this map.',
	            description_msgid='help_height',
	            i18n_domain='googlemaps',
	        )
	    ),
    LatLngField('center',
	        required=True,
	        widget=LatLngWidget(
	            label='Center',
	            label_msgid='label_center',
	            description='Center of this map.',
	            description_msgid='help_center',
	            i18n_domain='googlemaps',
	            size=12,
	        )
	    ),
    IntegerField('zoom',
	        default=15,
	        vocabulary=zoom_vocaburary,
	        widget=SelectionWidget(
	            label='Zoom',
	            label_msgid='label_zoom',
	            description='Select a zoom level.',
	            description_msgid='help_zoom',           
	            i18n_domain='googlemaps',
	        )
	    ),
    StringField('mapType',
	        vocabulary=(('HYBRID', 'Map+Satellite'),
                            ('ROADMAP', 'Map'),
                            ('SATELLITE', 'Satellite'),
                            ('TERRAIN', 'Terrain'),
                            ),
	        default='ROADMAP',
	        widget=SelectionWidget(
	            label='Map Type',
	            label_msgid='label_map_type',
	            description='Select a default map type.',
	            description_msgid='help_map_type',
	            i18n_domain='googlemaps',
	        ),
	    ),
    StringField('mapTypeControl',
	        vocabulary=(('DEFAULT', 'Default'),
                            ('HORIZONTAL_BAR', 'Horizontal bar'),
                            ('DROPDOWN_MENU', 'Dropdown menu'),
                            ('hide', 'hide'),
                            ),
	        default='DEFAULT',
	        widget=SelectionWidget(
	            label='Select mapType control type.',
	            label_msgid='label_map_type_control',
	            description_msgid='help_map_type_control',
	            i18n_domain='googlemaps',
	        ),
	        schemata='control settings',
	    ),
    StringField('navigationControl',
	        vocabulary=(('DEFAULT', 'Default'),
                            ('SMALL', 'Small'),
                            ('ZOOM_PAN', 'Zoom and Pan'),
                            ('ANDROID', 'Android'),
                            ('hide', 'hide'),
                            ),
	        default='DEFAULT',
	        widget=SelectionWidget(
	            label='Navigation Control',
	            label_msgid='label_navigation_control',
	            description='Select navigation control type.',
	            description_msgid='help_navigation_control',
	            i18n_domain='googlemaps',
	        ),
	        schemata='control settings',
	    ),
    ),)
gmap_schema = getattr(ATFolder, 'schema', Schema(())).copy() + schema.copy()
finalizeATCTSchema(gmap_schema)

class GMap(ATFolder):
    """
    For GMap
    """
    security = ClassSecurityInfo()
    interface.implements(IGMap)
    schema = gmap_schema
    
    meta_type = "GMap"
    _at_rename_after_creation = True

    # Don't show the display options
    def canSetDefaultPage(self):
        return False

registerType(GMap, PROJECTNAME)



from Products.CMFCore.utils import getToolByName

print "Migrate Float field to Latlng filed"
print ""
catalog = getToolByName(context, 'portal_catalog')
content = catalog.searchResults(portal_type = ['GMap', 'GMarker'])
for item in content:
    obj = item.getObject()
    if hasattr(obj, 'latitude') and hasattr(obj, 'longitude'):
        latlng = {'latitude': obj.latitude,
                  'longitude': obj.longitude}
        if obj.portal_type == 'GMap':
            print obj.portal_type, obj.absolute_url()
            obj.setCenter(latlng)
            obj.setScaleControl(True)
        if obj.portal_type == 'GMarker':
            print obj.portal_type, obj.absolute_url()
            obj.setPoint(latlng)

return printed



site_encoding =  context.plone_utils.getSiteEncoding()

def add_markers(js, items):
    for index in range(len(items)):
        marker = items[index].getObject()
        lat = marker.point['latitude']
        lng = marker.point['longitude']
        js.append('  var marker_%d = createMarker(map, %s, %s, "%d", "%s");' % (index, lat, lng, index, marker.title))

# create initialize method
def initialize_func(js):
    lat = context.center['latitude']
    lng = context.center['longitude']
    js.extend(('function initialize() {',
               '  var map = createMap("map_canvas", %s, %s, %s, "%s", "%s", "%s");' % (lat, lng, context.zoom, context.mapType, context.mapTypeControl, context.navigationControl),
               ))

    add_markers(js, context.getFolderContents())

    js.append('}')
    
js = ['<script type="text/javascript">']
initialize_func(js)
js.append('google.maps.event.addDomListener(window, "load", initialize);')
js.append('</script>')

return "\n".join(js)



#!/usr/bin/env python
# encoding: utf-8
"""
test_setup.py

Created by Manabu Terada on 2009-12-16.
Copyright (c) 2009 CMScom. All rights reserved.
"""
from Products.CMFCore.utils import getToolByName

import Products.ATGoogleMaps
import base

class TestInstall(base.TestCase):
    """ Install basic test """ 
    def afterSetUp(self):
        pass

    def testQuickInstall(self):
        qi = self.portal.portal_quickinstaller
        print '\n'.join(p['id'] for p in qi.listInstallableProducts())
        self.failUnless(1, len([p['title'] for p in qi.listInstallableProducts() 
                                        if p['title'] == 'ATGoogleMaps']))
        self.failUnless('ATGoogleMaps' in (p['id'] for p in qi.listInstallableProducts()))        
        qi.installProduct('ATGoogleMaps')
        self.failUnless(qi.isProductInstalled('ATGoogleMaps'))
        self.failUnless('ATGoogleMaps' not in ''.join(p['id'] for p in qi.listInstallableProducts()))        
    
class TestSkinInstall(base.TestCase):
    """  """
    def afterSetUp(self):
        qi = self.portal.portal_quickinstaller
        qi.installProduct('ATGoogleMaps')

    def testSkinLayersInstalled(self):
        self.skins = self.portal.portal_skins
        # print self.skins.objectIds()
        self.failUnless('ATGoogleMaps' in self.skins.objectIds())
        self.assertEqual(len(self.skins.ATGoogleMaps.objectIds()), 1)

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestInstall))
    suite.addTest(makeSuite(TestSkinInstall))
    return suite


from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.Archetypes.Registry import registerField
from Products.Archetypes.Field import ObjectField, Field
from Products.ATGoogleMaps.widget import LatLngWidget

import validator

class LatLngField(ObjectField):
    """A field that store latitude and longitude value"""
    __implements__ = ObjectField.__implements__

    _properties = Field._properties.copy()
    _properties.update({
        'type': 'latlng',
        'default': {},
        'size': 12,
        'default': None,
        'widget': LatLngWidget,
        'validators': ('LatLngValidator'),
        })

    security  = ClassSecurityInfo()
        
    security.declarePrivate('validate_required')
    def validate_required(self, instance, value, errors):
        try:
            float(value.latitude)
            float(value.longitude)
        except (ValueError, TypeError):
            result = False
        else:
            result = True
        return ObjectField.validate_required(self, instance, result, errors)

    security.declarePrivate('get')
    def get(self, instance, **kwargs):
        return ObjectField.get(self, instance, **kwargs)

    security.declarePrivate('set')
    def set(self, instance, value, **kwargs):
        if type(value) != type({}) and hasattr(value, 'keys'):
            new_value = {}
            new_value.update(value)
            value = new_value
            
        ObjectField.set(self, instance, value, **kwargs)

InitializeClass(LatLngField)

registerField(LatLngField,
              title="LatLng",
              description="Used to store longitude and longitude.",
              )




from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import StringWidget
from Products.Archetypes.Registry import registerWidget

class LatLngWidget(StringWidget):
    _properties = StringWidget._properties.copy()
    _properties.update({
        'macro' : "latlng_widget",
        })
    security = ClassSecurityInfo()

InitializeClass(LatLngWidget)

registerWidget(LatLngWidget,
               title='LatLng',
               description="Renders latitude and longitude fields.",
               used_for=('Products.ATGoogleMaps.field.LatLngField',)
               )



"""Common configuration constants
"""

PROJECTNAME = 'ATGoogleMaps'

ADD_PERMISSIONS = {
    # -*- extra stuff goes here -*-
    'GMap' : 'ATGoogleMaps: Add GMap',
    'GMarker' : 'ATGoogleMaps: Add GMarker',
}



from Globals import InitializeClass
from Products.validation import validation
from Products.validation.interfaces.IValidator import IValidator
from Products.CMFCore.utils import getToolByName

class LatLngValidator:
    """
    Latitude and Longitude validator. To be used with LatLngField.
    """

    __implements__ = IValidator

    def __init__(self, name, title='', description=''):
        self.name = name
        self.title = title
        self.description = description

    def __call__(self, value, *args, **kwargs):
        """
        Tests if the password values match. If value is just one string,
        only tests size.
        """

        try:
            latitude = float(value.latitude)
            longitude = float(value.longitude)
        except:
            return 1

        if latitude < -90 or latitude > 90:
            return 'Latitude is out of range.'
        if longitude < -180 or longitude > 180:
            return 'Longitude is out of range.'

        return 1

validation.register(LatLngValidator('LatLngValidator'))
